'use strict';

function MPInt() {
  const Cc = Components.classes;
  const Ci = Components.interfaces;

  const RegKeyClass = '@mozilla.org/windows-registry-key;1';
  const WRegKey = Ci.nsIWindowsRegKey;
  const HKCR = 0;
  const HKCU = 1;
  const HKLM = 2;

  const AppPaths = 'Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\';
  const MDProp = 'Software\\MetaProducts\\Mass Downloader';
  const DEProp = 'Software\\MetaProducts\\Download Express';
  const IQProp = 'Software\\MetaProducts\\Inquiry\\US';

  var Inited = false;
  var CPB, prefService, gprefs, prefs;

  this.IsMD = false;
  this.IsDE = false;
  this.IsOE = false;
  this.IsIQ = false;
  this.AccTypes = [];
  var RawAccTypes = '';
  var MDPath = '';
  var DEPath = '';
  var OEPath = '';
  var IQPath = '';

  function getRegistry() {
    let reg;
    if (RegKeyClass in Cc) {
      try {
        reg = Cc[RegKeyClass].createInstance(WRegKey);
      } catch(ex) {}
      if ((typeof(reg) == 'undefined') || (typeof(reg.open) != 'function'))
        reg = false;
    };
    return reg;
  };

  function getRegKey(reg, key) {
    switch (key) {
      case HKCR:
        return reg.ROOT_KEY_CLASSES_ROOT;
      case HKCU:
        return reg.ROOT_KEY_CURRENT_USER;
      case HKLM:
        return reg.ROOT_KEY_LOCAL_MACHINE;
    }
  };

  function getRegString(key, path, name) {
    let res = '';
    try {
      let reg = Cc[RegKeyClass].createInstance(WRegKey);
      reg.open(getRegKey(reg, key), path, WRegKey.ACCESS_READ);
      try {
        res = reg.readStringValue(name);
      } catch(ex) {}
      reg.close;
    } catch(ex) {}
    return res;
  };

  function setRegString(key, path, name, value) {
    try {
      let reg = Cc[RegKeyClass].createInstance(WRegKey);
      reg.open(getRegKey(reg, key), path, WRegKey.ACCESS_WRITE);
      try {
        reg.writeStringValue(name, value);
      } catch(ex) {}
      reg.close;
    } catch(ex) {}
  };

  this.SavePrefs = function() {
    try {
      prefService.savePrefFile(null);
    } catch (ex) {};
  }

  this.getPref = function(name, def) {
    try {
      switch (prefs.getPrefType(name)) {
        case CPB.PREF_STRING:
          return prefs.getCharPref(name);
        case CPB.PREF_INT:
          return prefs.getIntPref(name);
        case CPB.PREF_BOOL:
          return prefs.getBoolPref(name);
      }
    } catch(ex) {}
    return def;
  }

  this.setPref = function(name, val) {
    switch (typeof(val)) {
      case 'string':
        prefs.setCharPref(name, val);
        break;
      case 'boolean':
        prefs.setBoolPref(name, val);
        break;
      case 'number':
        prefs.setIntPref(name, val);
        break;
    }
  }

  function ConvertAccTypes(s) {
    RawAccTypes = s;
    let t = s.toLowerCase().replace(/\s/g, '').split(',');
    let i = 0;
    while (i < t.length) {
      if (!t[i])
        t.splice(i, 1);
      else
        i++;
    };
    return t.sort();
  }

  this.SaveExts = function() {
    RawAccTypes = this.AccTypes.join(',');
    this.setPref('AccTypes', RawAccTypes);
  };

  this.GetExts = function() {
    var s = this.getPref('AccTypes', '');
    if (s != RawAccTypes)
      this.AccTypes = ConvertAccTypes(s);
    return this.AccTypes;
  }

  this.AddExt = function(ext) {
    if (this.InTypes(ext) == -1) {
      this.AccTypes.push(ext);
      this.AccTypes.sort();
      this.SaveExts();
      this.SavePrefs();
    }
  }

  this.InTypes = function(ext) {
    this.GetExts();
    let len = this.AccTypes.length;
    for (let i = 0; i < len; i++) {
      if (this.AccTypes[i] == ext)
        return i;
    }
    return -1;
  }

  function FileExists(fname) {
    let f = Cc['@mozilla.org/file/local;1'].createInstance(Ci.nsILocalFile);
    f.initWithPath(fname);
    return f.exists();
  }

  function RunIt(exepath, args) {
    try {
      var f = Cc['@mozilla.org/file/local;1'].createInstance(Ci.nsILocalFile);
      var pr = Cc['@mozilla.org/process/util;1'].createInstance(Ci.nsIProcess);
      f.initWithPath(exepath);
      if (!f.exists()) {
        alert('Executable "' + exepath + '" does not exist.');
      } else {
        pr.init(f);
        pr.run(false, args, args.length);
      }
    } catch(ex) {
      alert('Cannot run executable: ' + ex);
    }
  }

  function GetTitle() {
    var title = '';
    var wind = gBrowser.contentWindow;
    var sel = wind.getSelection();
    if (sel)
      title = sel.toString().trim();
    if (!title)
      title = wind.document.title.trim();
    if (!title) {
      var s1 = wind.location.href.trim();
      var ar = s1.split('/');
      if (ar.length > 0) {
        title = ar[ar.length - 1];
        ar = title.split('.');
        if (ar.length > 0)
          title = ar[0];
      };
      if (!title)
        title = s1;
    };
    if (title) {
      title = title.replace(/\t/g, ' ');
      title = title.replace(/\r/g, '\n');
      var ar = title.split('\n');
      if (ar.length > 0)
        title = ar[0];
      if (title.length > 140) {
        title = title.substring(0, 140);
        var i = title.lastIndexOf(' ');
        var j = title.lastIndexOf('.');
        if ((j != -1) && (j > i))
          i = j;
        if (i != -1)
          title = title.substring(0, i);
      };
    };
    return title;
  };

  function GetComment() {
    var metas = gBrowser.contentWindow.document.getElementsByTagName('meta');
    for (let i = 0; i < metas.length; i++) {
      if (metas[i].hasAttribute('name') && metas[i].hasAttribute('content')
       && (metas[i].getAttribute('name').toUpperCase() == 'DESCRIPTION'))
        return metas[i].getAttribute('content');
    };
    return '';
  };

  this.DownloadMDURL = function(url, refe, Comment, fn, mime) {
    var links = [{url: url || '', comment: Comment || '', cookie: ''}];
    GetCookies(links);
    Components.utils.import('resource://mpint/winint.js');
    WinInt.DownloadMDURL(links[0].url, refe, links[0].comment,
     links[0].cookie, fn, mime);
  };

  var DownloadLinks = function(refe, links) {
    if (links.length > 0) {
      GetCookies(links);
      Components.utils.import('resource://mpint/winint.js');
      WinInt.DownloadMDURLs(refe, links);
    };
  };

  this.DownloadMDSel = function(refe) {
    var links = this.GrabSelLinks();
    DownloadLinks(refe, links);
  };

  this.DownloadMDAll = function(refe) {
    var links = GrabAllLinks(CurDocument());
    DownloadLinks(refe, links);
  };

  this.DownloadDEURL = function(url, refe, Comment, fn, mime) {
    var links = [{url: url || '', comment: Comment || '', cookie: ''}];
    GetCookies(links);
    Components.utils.import('resource://mpint/winint.js');
    WinInt.DownloadDEURL(links[0].url, refe, links[0].comment,
     links[0].cookie, fn, mime);
  };

  this.DownloadOEURL = function(url, refe) {
    var args = new Array();
    var s = '/new=url=' + url;
    if (refe)
      s += ',url=Referer=' + refe;
    s += ',source=MPInt';
    args.push(s);
    args.push('/NoWizard');
    RunIt(this.OEPath, args);
  }

  this.DownloadIQURL = function(url, LinkText) {
    if (url) {
      var Title = '';
      var Comment = '';

      if (LinkText) {
        Title = LinkText
      } else {
        Title = GetTitle();
        Comment = GetComment();
      }

      var links = [{url: url || '', comment: '', cookie: ''}];
      GetCookies(links);
      var Cookie = links[0].cookie || '';

      setRegString(HKCU, IQProp, 'Title', Title);
      setRegString(HKCU, IQProp, 'Comment', Comment);
      setRegString(HKCU, IQProp, 'Cookie', Cookie);

      var isImg = true;
      if (gprefs.prefHasUserValue('permissions.default.image'))
        isImg = gprefs.getIntPref('permissions.default.image') != 2

      var args = new Array();
      if (this.getPref('ShowIQDlg', false)) {
        if (this.getPref('ShowIQDlg2', 0) == 0)
          args.push('-db')
        else
          args.push('-da');
      } else
        args.push('-d');
      args.push('0');
      args.push(url);
      args.push(window.innerWidth);
      args.push(window.innerHeight);
      if (isImg)
        args.push('1')
      else
        args.push('0');
      if (this.getPref('IQToDisk', false))
        args.push('-disk');

      RunIt(this.IQPath, args);
    };
  };

  this.getRefe = function() {
    try {
      return content.location;
    } catch(ex) {
      try {
        return CurWindow().location;
      } catch(ex) {};
    };
    return null;
  };

  this.SetEnabled = function(elm, enab) {
    if (enab)
      elm.removeAttribute('disabled')
    else
      elm.setAttribute('disabled', true);
  };

  function CurWindow() {
    return document.commandDispatcher.focusedWindow;
  };

  function CurDocument() {
    return CurWindow().document;
  };

  this.IsSel = function() {
    return !CurWindow().getSelection().isCollapsed;
  };

  this.IsLinks = function() {
    var doc = CurDocument();
    if (doc.links.length > 0) {
      return true;
    } else {
      var frames = doc.getElementsByTagName('frame');
      for (let i = 0; i < frames.length; i++) {
        try {
          if (this.IsLinks(frames(i).document))
            return true;
        } catch(ex) {};
      };
      frames = doc.getElementsByTagName('iframe');
      for (let i = 0; i < frames.length; i++) {
        try {
          if (this.IsLinks(frames(i).document))
            return true;
        } catch(ex) {};
      };
    };
    return false;
  };

  function GetCookies(links) {
    var i, pos, iter;
    var l, host;
    const hostCookies = {};
    const domainCookies = {};
    var objCookie;
    var cookieHost, cookieTable, tmpCookie;

    iter = Cc['@mozilla.org/cookiemanager;1']
     .getService(Ci.nsICookieManager).enumerator;
    if (iter) {
      for (i = 0; i < links.length; i++) {
        l = links[i];
        host = l.url.match(/https?:\/\/([^\/]+\.[^\/]+)/i);
        if (host) {
          host = host[1];
          pos = host.indexOf('@');
          if (pos > -1)
            host = host.substring(pos + 1);
          l.host = host;
          hostCookies[host] = '';
        } else
          l.host = null;
      };

      while (iter.hasMoreElements()) {
        objCookie = iter.getNext();
        if (objCookie instanceof Ci.nsICookie) {
          cookieHost = objCookie.host;
          if (cookieHost.charAt(0) == '.') {
            cookieHost = cookieHost.substring(1);
            cookieTable = domainCookies;
            if (typeof(tmpCookie = domainCookies[cookieHost]) != 'string')
              tmpCookie = '';
          } else {
            if (typeof(tmpCookie = hostCookies[cookieHost]) != 'string')
              continue;
            cookieTable = hostCookies;
          };
          cookieTable[cookieHost] = tmpCookie.concat(objCookie.name
           + '=' + objCookie.value + '; ');
        };
      };

      for (cookieHost in hostCookies) {
        for (host = cookieHost; (pos = host.indexOf('.')) >= 0; ) {
          if (tmpCookie = domainCookies[host])
            hostCookies[cookieHost] += tmpCookie;
          host = host.substring(pos + 1);
        };
      };

      for (i = 0; i < links.length; i++) {
        l = links[i];
        host = l.host;
        if (host)
          l.cookie = hostCookies[host] || '';
        else
          l.cookie = '';
      };
    };
  };

  function GetTextUnder(root) {
    var text = '';
    var node = root.firstChild;
    var depth = 1;
    while (node && depth > 0) {
      // See if this node is text.
      if (node.nodeType == Node.TEXT_NODE) {
        // Add this text to our collection.
        text += ' ' + node.data;
      } else if (node instanceof HTMLImageElement) {
        // If it has an alt= attribute, use that.
        var altText = node.getAttribute('alt');
        if (altText && altText != '') {
          text = altText;
          break;
        }
      }
      // Find next node to test.
      // First, see if this node has children.
      if (node.hasChildNodes()) {
        // Go to first child.
        node = node.firstChild;
        depth++;
      } else {
        // No children, try next sibling.
        if (node.nextSibling) {
          node = node.nextSibling;
        } else {
          // Last resort is our next oldest uncle/aunt.
          node = node.parentNode.nextSibling;
          depth--;
        }
      }
    }
    // Strip leading whitespace.
    text = text.replace(/^\s+/, '');
    // Strip trailing whitespace.
    text = text.replace(/\s+$/, '');
    // Compress remaining whitespace.
    text = text.replace(/\s+/g, ' ');
    return text;
  };

  function linkText(link, url) {
    var text = GetTextUnder(link);
    if (!text || !text.match(/\S/)) {
      text = link.getAttribute('title');
      if (!text || !text.match(/\S/)) {
        text = link.getAttribute('alt');
        if (!text || !text.match(/\S/))
          text = url;
      };
    };
    return text;
  };

  function GrabLinks(links, doc, sels) {
    if (doc) {
      var len = doc.links.length;
      for (let i = 0; i < len; i++) {
        var l = doc.links[i];
        var url = l.href;
        var s = url.toLowerCase();
        if (!/^(mailto|news):/i.test(s)) {
          var b = true;
          if (sels)
            b = sels.containsNode(l, true);
          var comm = linkText(l, url);
          if (b) {
            for (let j = 0; j < links.length; j++) {
              if (url == links[j].url) {
                b = false;
                if (links[j].comment == links[j].url)
                  links[j].comment = comm;
                break;
              };
            };
          };
          if (b)
            links.push({url: url, comment: comm, cookie: ''});
        };
      };
    };
  };

  function GrabTextLinks(links, sels) {
    function AddURL(url) {
      url = url.replace(/(.+)[.,!?]?/, '$1');
      var b = true;
      for (let j = 0; j < links.length; j++) {
        if (url == links[j].url) {
          b = false;
          break;
        };
      };
      if (b)
        links.push({url: url, comment: url, cookie: ''});
    };

    var u = sels.toString().match(/^\s*((?:(?:https?|ftp):\/\/)?(?:[a-z0-9](?:[-\w]*[a-z0-9])?\.)+[a-z]{2,}(?::\d+)?(?:\/[^\s]*)?)[.,!?]?\s*$/i);
    if (u) {
      AddURL(u[1])
    } else {
      u = sels.toString().match(/((?:(?:https?|ftp):\/\/[-\w]+(?:\.\w[-\w]*)+)[^\s]*)|((?:[a-z0-9](?:[-\w]*[a-z0-9])?\.)+[a-z]{2,}(?::\d+)?\/[^\s]*)/gi);
      if (u) {
        for (var j = 0; j < u.length; j++)
          AddURL(u[j]);
      };
    };
  };

  function GrabAllLinks(doc) {
    var frames;
    var links = [];
    GrabLinks(links, doc);
    frames = doc.getElementsByTagName('frame');
    for (let i = 0; i < frames.length; i++) {
      try {
        links = links.concat(GrabAllLinks(frames(i).document));
      } catch(ex) {};
    };
    frames = doc.getElementsByTagName('iframe');
    for (let i = 0; i < frames.length; i++) {
      try {
        links = links.concat(GrabAllLinks(frames(i).document));
      } catch(ex) {};
    };
    return links;
  };

  this.GrabSelLinks = function() {
    var links = [];
    var sels = CurWindow().getSelection();
    if (sels) {
      GrabLinks(links, CurDocument(), sels);
      GrabTextLinks(links, sels);
    };
    return links;
  };

  this.init = function() {
    if (Inited)
      return;
    
    Inited = true;
    
    CPB = Ci.nsIPrefBranch;
    prefService = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService);
    gprefs = prefService.getBranch('');
    prefs = prefService.getBranch('mpint.');

    if (getRegistry()) {
      try {
        this.MDPath = getRegString(HKLM, AppPaths + 'massdown.exe', '');
      } catch (ex) {};
      this.IsMD = this.MDPath && FileExists(this.MDPath);
      if (!this.IsMD) {
        try {
          this.MDPath = getRegString(HKLM, AppPaths + 'pdownloadmanager.exe', '');
        } catch (ex) {};
        this.IsMD = this.MDPath && FileExists(this.MDPath);
      }

      try {
       this.DEPath = getRegString(HKCU, DEProp, 'ProgramPath');
      } catch (ex) {};
      this.IsDE = this.DEPath && FileExists(this.DEPath);

      try {
        this.OEPath = getRegString(HKLM, AppPaths + 'oe.exe', '');
      } catch (ex) {};
      this.IsOE = this.OEPath && FileExists(this.OEPath);

      try {
        this.IQPath = getRegString(HKLM, AppPaths + 'iqls.exe', '');
      } catch (ex) {};
      this.IsIQ = this.IQPath && FileExists(this.IQPath);
    };

    var DefDL = this.getPref('DefDL', 'MD');
    if ((DefDL == 'MD') && (!this.IsMD) && this.IsDE) {
      DefDL = 'DE';
      this.setPref('DefDL', DefDL);
    } else if ((DefDL == 'DE') && (!this.IsDE) && this.IsMD) {
      DefDL = 'MD';
      this.setPref('DefDL', DefDL);
    };

    if ((!this.getPref('IsInit', false)) && (this.IsMD || this.IsDE)) {
      this.setPref('IsInit', true);
      var s = '';
      try {
        if (DefDL == 'MD')
          s = getRegString(HKCU, MDProp, 'AcceptingTypes');
        else
          s = getRegString(HKCU, DEProp, 'AcceptingTypes');
      } catch (ex) {};
      this.AccTypes = ConvertAccTypes(s);
      this.SaveExts();
      this.SavePrefs();
    };
  };
};

if (!MPIntComm) {
  var MPIntComm = new MPInt();
};
